package com.base.helper;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.atomic.AtomicLong;

/**
 * @author mzl
 * @desc
 */
public class IDGenerator {
    public static final     Logger     LOGGER            = LoggerFactory.getLogger(IDGenerator.class);
    private static final    int        BITS_TO_MOVE      = 0x0c;
    private static final    int        NODE_BITS_TO_MOVE = 0x38;
    private static final    long       MASK_TIME         = 0xffffffffff0L;
    private static final    long       ID_MASK           = 0xffffL;
    private static final    long       TIME_ID_MASK      = 0xffffffffffff0000L;
    private static final    AtomicLong COUNTER           = new AtomicLong(0);
    private static          boolean    WRAPPED           = false;
    private static volatile long       tmMark;

    static {
        IDGenerator.refresh();
    }

    private IDGenerator() {
    }

    public static long generateID() {
        long idReturn = COUNTER.incrementAndGet();

        if ((idReturn & IDGenerator.ID_MASK) == 0) {
            final long timePortion = idReturn & IDGenerator.TIME_ID_MASK;
            if (timePortion >= IDGenerator.newTM()) {
                WRAPPED = true;
            } else {
                tmMark = timePortion;
            }
        }

        if (WRAPPED) {
            throw new IllegalStateException(
                    "The IdGenerator is being overlaped, and it needs revision as the system generated more than " +
                            IDGenerator.ID_MASK + " ids per 16 milliseconds which exceeded the IDgenerator limit");
        }

        return idReturn;
    }

    public static String generateIDStr() {
        return IDGenerator.generateID() + "";
    }

    public synchronized static void refresh() {
        long oldTm = tmMark;
        long newTm = IDGenerator.newTM();

        while (newTm <= oldTm) {
            newTm = IDGenerator.newTM();
        }
        tmMark = newTm;
        COUNTER.set(tmMark);
    }

    public static String currentInfo() {
        long currentCounter = COUNTER.get();
        return "IdGenerator(tmMark=" + IDGenerator.hex(tmMark) + ", CurrentCounter = " + currentCounter +
                ", HexCurrentCounter = " + IDGenerator.hex(currentCounter) + ")";
    }

    private static long newTM() {
        return (EnviromentUtils.NODE_ID << IDGenerator.NODE_BITS_TO_MOVE) |
                ((System.currentTimeMillis() & IDGenerator.MASK_TIME) << IDGenerator.BITS_TO_MOVE);
    }

    private static String hex(final long x) {
        return String.format("%1$X", x);
    }

}
