package com.pixelai.api.chat.utils;

import com.volcengine.ark.runtime.model.completion.chat.ChatCompletionRequest;
import com.volcengine.ark.runtime.model.completion.chat.ChatMessage;
import com.volcengine.ark.runtime.model.completion.chat.ChatMessageRole;
import com.volcengine.ark.runtime.service.ArkService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class VolcengineChatUtil {

    @Value("${volcengine.ArkApiKey}")
    private String arkApiKey;
    @Value("${volcengine.Model}")
    private String model;


    public Map<String, Object> doRequest(String message) {
        // 创建ArkService实例
        ArkService arkService = ArkService.builder().apiKey(arkApiKey).build();

        // 初始化消息列表
        List<ChatMessage> chatMessages = new ArrayList<>();
        ChatMessage userMessage;
        // 创建用户消息
        if(message.length()>0){
            userMessage = ChatMessage.builder()
                    .role(ChatMessageRole.USER) // 设置消息角色为用户
                    .content("请你丰富提示词，帮助用户更好的使用视觉大模型生成图片。文本要求两百字以内，格式要求不要换行，提示词如下："+message) // 设置消息内容
                    .build();
        }else {
            userMessage = ChatMessage.builder()
                    .role(ChatMessageRole.USER) // 设置消息角色为用户
                    .content("请你随机生成一张图片的提示词，帮助用户更方便的使用视觉大模型生成图片。文本要求两百字以内，格式要求不要换行。") // 设置消息内容
                    .build();
        }

        // 将用户消息添加到消息列表
        chatMessages.add(userMessage);

        // 创建聊天完成请求
        ChatCompletionRequest chatCompletionRequest = ChatCompletionRequest.builder()
                .model(model)// 需要替换为您的推理接入点ID
                .messages(chatMessages) // 设置消息列表
                .build();
        Map response = new HashMap();
        // 发送聊天完成请求并打印响应
        try {
            // 获取响应并打印每个选择的消息内容
//            arkService.createChatCompletion(chatCompletionRequest)
//                    .getChoices()
//                    .forEach(choice -> System.out.println(choice.getMessage().getContent()));
            // 获取响应并存储每个选择的消息内容
            arkService.createChatCompletion(chatCompletionRequest)
                    .getChoices()
                    .forEach(choice -> response.put("body",choice.getMessage().getContent()));
//            response.put("body",arkService.createChatCompletion(chatCompletionRequest)
//                    .getChoices());
//            System.out.println("\n\nbody:"+response.get("body"));
        } catch (Exception e) {
            response.put("body","请求失败: " + e.getMessage());
            System.out.println("请求失败: " + e.getMessage());
        } finally {
            // 关闭服务执行器
            arkService.shutdownExecutor();
        }
        return response;
    }

}
