package com.pixelai.api.component.controller;

import com.base.annotation.Unsecured;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.component.entity.CpBanner;
import com.pixelai.api.component.service.CpBannerService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-10-26
 */
@Api(tags = "轮播图管理")
@RestController
@RequestMapping("/cpBanner")
public class CpBannerController extends BaseController {

        private static final Logger       LOG = LoggerFactory.getLogger(CpBannerController.class);
        @Autowired
        private              CpBannerService entityService;

        @SysLog(action = "findByPage", value = "分页查询轮播图")
        @ApiOperation(value = "分页查询轮播图", notes = "分页查询轮播图")
        @RequestMapping(method = RequestMethod.GET)
        @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
                @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
        public Result<IPage<CpBanner>> findByPage(final CpBanner example, final Page page) {
            IPage<CpBanner>        records    = entityService.page(example,page);
            return new Result(true, records);
        }

        @SysLog(action = "delete", value = "删除轮播图")
        @ApiOperation(value = "删除轮播图")
        @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
        @ApiImplicitParam(name = "id", value = "轮播图ID", required = true, paramType = "path")
        public Result delete(@PathVariable final Integer id) {
            try {
                CpBanner cpBanner=entityService.getById(id);
                cpBanner.setState("f");
                entityService.updateById(cpBanner);
                return new Result(true, "成功删除轮播图", null);
            } catch (DataIntegrityViolationException e) {
                LOG.error("删除轮播图失败", e);
                return new Result(false, "删除轮播图失败", "该轮播图不能删除，存在其他关联数据");
            } catch (Exception e) {
                LOG.error("删除轮播图失败", e);
                return new Result(false, "删除轮播图失败", e.getMessage());
            }
        }

        @SysLog(action = "one", value = "查询单个轮播图")
        @ApiOperation(value = "查询单个轮播图")
        @RequestMapping(value = "{id}", method = {RequestMethod.GET})
        @ApiImplicitParam(name = "id", value = "轮播图ID", required = true, paramType = "path")
        public Result one(@PathVariable final Integer id) {
            try {
                CpBanner entity = entityService.getById(id);
                return new Result(true, entity);
            } catch (Exception e) {
                LOG.error("查询单个轮播图失败", e);
                return new Result(false, new CpBanner());
            }
        }

        @SysLog(action = "add", value = "添加轮播图")
        @ApiOperation(value = "添加轮播图")
        @RequestMapping(method = {RequestMethod.POST})
        public Result add(@Valid @RequestBody final CpBanner entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "保存轮播图失败", errorMsg, map);
                } else {
                    entity.setCreatetime(new Date());
                    entity.setState("t");
                    entityService.save(entity);
                    return new Result(true, "成功保存轮播图", null);
                }
            } catch (Exception e) {
                LOG.error("添加轮播图失败", e);
                return new Result(false, "保存轮播图失败", e.getMessage());
            }
        }

        @SysLog(action = "update", value = "修改轮播图")
        @ApiOperation(value = "修改轮播图")
        @RequestMapping(method = {RequestMethod.PUT})
        public Result update(@Valid @RequestBody final CpBanner entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "修改轮播图失败", errorMsg, map);
                } else {
                    if (null == entity.getId()) {
                         throw new RuntimeException("id不能为空");
                    }
                    entityService.updateById(entity);
                    return new Result(true, "成功修改轮播图", null);
                }
            } catch (Exception e) {
                LOG.error("修改轮播图失败", e);
                return new Result(false, "修改轮播图失败", e.getMessage());
            }
        }

        @ApiOperation(value = "全部轮播图")
        @RequestMapping(value = "all", method = RequestMethod.GET)
        @Unsecured
        public Result<List<CpBanner>> all() {
            List<CpBanner> entitys = entityService.lambdaQuery()
                    .eq(CpBanner::getState,"t")
                    .orderByAsc(CpBanner::getOrderNum)
                    .list();
            if (null != entitys) {
                return new Result<>(true, entitys);
            }
            return new Result<>(true, Collections.emptyList());
        }

}

