package com.pixelai.api.component.controller;

import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.component.entity.CpMessage;
import com.pixelai.api.component.service.CpMessageService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-10-27
 */
@Api(tags = "留言板")
@RestController
@RequestMapping("/cpMessage")
public class CpMessageController extends BaseController {

        private static final Logger       LOG = LoggerFactory.getLogger(CpMessageController.class);
        @Autowired
        private CpMessageService entityService;

        @SysLog(action = "findByPage", value = "分页查询留言")
        @ApiOperation(value = "分页查询留言", notes = "分页查询留言")
        @RequestMapping(method = RequestMethod.GET)
        @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
                @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
        public Result<IPage<CpMessage>> findByPage(final CpMessage example, final Page page) {
            IPage<CpMessage>        records    = entityService.page(example,page);
            return new Result(true, records);
        }

        @SysLog(action = "delete", value = "删除留言")
        @ApiOperation(value = "删除留言")
        @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
        @ApiImplicitParam(name = "id", value = "留言ID", required = true, paramType = "path")
        public Result delete(@PathVariable final Integer id) {
            try {
                entityService.removeById(id);
                return new Result(true, "成功删除留言", null);
            } catch (DataIntegrityViolationException e) {
                LOG.error("删除留言失败", e);
                return new Result(false, "删除留言失败", "该留言不能删除，存在其他关联数据");
            } catch (Exception e) {
                LOG.error("删除留言失败", e);
                return new Result(false, "删除留言失败", e.getMessage());
            }
        }

        @SysLog(action = "one", value = "查询单个留言")
        @ApiOperation(value = "查询单个留言")
        @RequestMapping(value = "{id}", method = {RequestMethod.GET})
        @ApiImplicitParam(name = "id", value = "留言ID", required = true, paramType = "path")
        public Result one(@PathVariable final Integer id) {
            try {
                CpMessage entity = entityService.getById(id);
                return new Result(true, entity);
            } catch (Exception e) {
                LOG.error("查询单个留言失败", e);
                return new Result(false, new CpMessage());
            }
        }

        @SysLog(action = "add", value = "添加留言")
        @ApiOperation(value = "添加留言")
        @RequestMapping(method = {RequestMethod.POST})
        public Result add(@Valid @RequestBody final CpMessage entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "保存留言失败", errorMsg, map);
                } else {
                    entityService.save(entity);
                    return new Result(true, "成功保存留言", null);
                }
            } catch (Exception e) {
                LOG.error("添加留言失败", e);
                return new Result(false, "保存留言失败", e.getMessage());
            }
        }

        @SysLog(action = "update", value = "修改留言")
        @ApiOperation(value = "修改留言")
        @RequestMapping(method = {RequestMethod.PUT})
        public Result update(@Valid @RequestBody final CpMessage entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "修改留言失败", errorMsg, map);
                } else {
                    if (null == entity.getId()) {
                         throw new RuntimeException("id不能为空");
                    }
                    entityService.updateById(entity);
                    return new Result(true, "成功修改留言", null);
                }
            } catch (Exception e) {
                LOG.error("修改留言失败", e);
                return new Result(false, "修改留言失败", e.getMessage());
            }
        }

        @ApiOperation(value = "全部留言")
        @RequestMapping(value = "all", method = RequestMethod.GET)
        public Result<List<CpMessage>> all(CpMessage example) {
            List<CpMessage> entitys = entityService.list(example);
            if (null != entitys) {
                return new Result<>(true, entitys);
            }
            return new Result<>(true, Collections.emptyList());
        }

}

