package com.pixelai.api.component.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.context.Context;
import com.base.context.ContextHolder;
import com.pixelai.api.component.dao.CpLabelMapper;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.dao.CpPhotoMapper;
import com.pixelai.api.component.service.CpPhotoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pixelai.api.component.vo.CpLabelVo;
import com.pixelai.api.component.vo.CpPhotoVo;
import com.pixelai.utils.FileUtils;
import com.pixelai.utils.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author gjj
 * @since 2024-10-26
 */
@Service
public class CpPhotoServiceImpl extends ServiceImpl<CpPhotoMapper, CpPhoto> implements CpPhotoService {
    @Autowired
    private CpLabelMapper cpLabelMapper;

    @Override
    public CpPhotoVo savePhoto(CpPhotoVo cpPhotoVo) {
        Context context = ContextHolder.currentContext();
        //查找对应模块名称和标签名称的标签
        CpLabelVo cpLabelVo=cpLabelMapper.findByName(cpPhotoVo.getModuleName(),cpPhotoVo.getLabelName());
        if(cpLabelVo==null){
            return null;
        }
        cpPhotoVo.setUserid(context.currentUser().getId());
        cpPhotoVo.setCreatetime(new Date());
        cpPhotoVo.setCpLabelId(cpLabelVo.getLabelId());
        cpPhotoVo.setState("t");
        cpPhotoVo.setSize(FileUtils.getFileLength(cpPhotoVo.getPath())+"");
        this.save(cpPhotoVo);
        return  cpPhotoVo;
    }

    @Override
    public CpPhotoVo updatePhoto(CpPhotoVo cpPhotoVo) {
        Context context = ContextHolder.currentContext();
        //查找对应模块名称和标签名称的标签
        CpLabelVo cpLabelVo=cpLabelMapper.findByName(cpPhotoVo.getModuleName(),cpPhotoVo.getLabelName());
        if(cpLabelVo==null){
            return null;
        }
        cpPhotoVo.setUserid(context.currentUser().getId());
        cpPhotoVo.setCpLabelId(cpLabelVo.getLabelId());
        cpPhotoVo.setState("t");
        cpPhotoVo.setSize(FileUtils.getFileLength(cpPhotoVo.getPath())+"");
        this.updateById(cpPhotoVo);
        return cpPhotoVo;
    }

    @Override
    public Boolean deletePhotoById(Integer id) {
        return this.lambdaUpdate().set(CpPhoto::getState,"f").eq(CpPhoto::getId, id).update();
    }

    @Override
    public List<CpPhoto> list(CpPhoto example) {
        return this.list(buildWrapper(example));
    }

    @Override
    public IPage<CpPhoto> page(CpPhoto example, IPage page) {
        return this.page(page,buildWrapper(example));
    }

    @Override
    public Page<CpPhotoVo> findByPage(CpPhotoVo example, Page page) {
        page.setOptimizeCountSql(false);
        return page.setRecords(this.baseMapper.findByExample(buildCpPhotoVoWrapper(example), page));
    }

    /**
     * 构建查询
     *
     * @param example
     * @return
     */
    private QueryWrapper<CpPhotoVo> buildCpPhotoVoWrapper(CpPhotoVo example) {
        QueryWrapper<CpPhotoVo> wrapper     = new QueryWrapper<>();
        wrapper.like(StringUtil.isNotBlank(example.getName()),"name",example.getName())
                .like(StringUtil.isNotBlank(example.getLabelName()),"label_name",example.getLabelName())
                .like(StringUtil.isNotBlank(example.getModuleName()),"module_name",example.getModuleName())
                .eq(example.getId()!=null,"id",example.getId())
                .eq("state","t")
                .orderByDesc("createtime");
        return wrapper;
    }

    /**
     * 构建查询
     *
     * @param example
     * @return
     */
    private QueryWrapper<CpPhoto> buildWrapper(CpPhoto example) {
        QueryWrapper<CpPhoto> wrapper     = new QueryWrapper<>();
        return wrapper;
    }

}
