package com.pixelai.api.domain.apy;

import com.pixelai.api.pa.entity.PaConsumption;
import com.pixelai.api.pa.entity.PaOrder;
import com.pixelai.api.pa.entity.PaVipCurrency;
import com.pixelai.api.pa.entity.enums.PayState;
import com.pixelai.api.pa.service.PaConsumptionService;
import com.pixelai.api.pa.service.PaOrderService;
import com.pixelai.api.pa.service.PaVipCurrencyService;
import com.pixelai.api.pa.service.PaVipService;
import com.pixelai.config.WeChatConfig;
import com.pixelai.utils.WeChatUtil;
import com.wechat.pay.java.core.exception.HttpException;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.core.exception.ValidationException;
import com.wechat.pay.java.core.http.*;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.util.GsonUtil;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.model.Payer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.wechat.pay.java.service.partnerpayments.h5.model.Transaction;
import com.wechat.pay.java.service.payments.h5.H5Service;
import com.wechat.pay.java.service.payments.h5.model.*;
import eu.bitwalker.useragentutils.UserAgent;
import io.undertow.util.MalformedMessageException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.impl.client.HttpClientBuilder;
import org.springframework.transaction.annotation.Transactional;
import java.util.*;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class OrderPayAppService {
    @Autowired
    private WeChatConfig weChatConfig;
    @Resource
    private NotificationParser notificationParser;
    @Resource
    private H5Service h5Service;
//    @Resource
//    private JsapiService jsapiService;
    @Resource
    private PaOrderService paOrderService;
    @Resource
    private PaVipCurrencyService paVipCurrencyService;
    @Resource
    private PaConsumptionService paConsumptionService;

    public PaOrder h5Pay(PaOrder order,HttpServletRequest request) {
        List<String> buyTypes = new ArrayList<>(Arrays.asList("1","2","3","4","5","6"));
        if (order.getBuyType()==null||!buyTypes.contains(order.getBuyType())) {
            throw new RuntimeException("购买类型错误");
        }
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        PrepayRequest prepayRequest = new PrepayRequest();
        Amount amount = new Amount();
        amount.setTotal((order.getAmount().multiply(new BigDecimal("100"))).intValue());
        prepayRequest.setAmount(amount);
        prepayRequest.setAppid(weChatConfig.getAppId());
        prepayRequest.setMchid(weChatConfig.getMerchantId());
        prepayRequest.setDescription("钻石购买");
        prepayRequest.setNotifyUrl(weChatConfig.notifyUrl);
        String tradeNo = WeChatUtil.generateTradeNumber();
        prepayRequest.setOutTradeNo(tradeNo);
        SceneInfo sceneInfo = new SceneInfo();
        sceneInfo.setPayerClientIp(weChatConfig.getAddressIp());
        H5Info h5Info = new H5Info();
        h5Info.setType(UserAgent.parseUserAgentString(request.getHeader("User-Agent")).getOperatingSystem().getName());
        sceneInfo.setH5Info(h5Info);
        prepayRequest.setSceneInfo(sceneInfo);
        // 调用下单方法，得到应答
        PrepayResponse response;
        try {
            response = h5Service.prepay(prepayRequest);
            //预支付成功，创建预支付订单
            order.setOrderNum(tradeNo);
            order.setPayStatus(PayState.TO_BE_PAID);
            order.setCreateTime(new Date());
            order.setState("t");
            order.setH5Url(response.getH5Url());
            paOrderService.save(order);
            return order;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Transactional
    public void payNotify(HttpServletRequest request) throws Exception{
        Logger logger = LoggerFactory.getLogger(getClass()); // 获取日志记录器
        Transaction transaction;
        try {
            System.out.println(request);
            // 打印请求信息，便于调试
            logger.info("Received payment notification request: {}", request);
            transaction = notificationParser.parse(WeChatUtil.handleNodifyRequestParam(request), Transaction.class);
            logger.info("Parsed transaction: {}", transaction);
            if (transaction.getTradeState() == Transaction.TradeStateEnum.SUCCESS) {
                PaOrder order = paOrderService.getOrderByTradeNo(transaction.getOutTradeNo());
                if (Objects.isNull(order)) {
                    logger.warn("Order not found for tradeNo: {}", transaction.getOutTradeNo());
                    return;
                }
                //校验订单状态：若订单已支付则直接返回成功
                if (Objects.equals(order.getPayStatus(), PayState.HAVE_TO_PAY)) {
                    logger.info("Order already paid, skipping further processing. Order ID: {}", order.getId());
                    return;
                }
                //支付成功-修改订单状态
                order.setPayStatus(PayState.HAVE_TO_PAY);
                order.setResult(transaction.getTradeStateDesc());
                paOrderService.updateById(order);
                logger.info("Order status updated to HAVE_TO_PAY. Order ID: {}", order.getId());
                //支付成功-充值钻石
                PaVipCurrency currency = paVipCurrencyService.findByUserId(order.getMemberId());
                currency.setState("t");
                if (order.getBuyType().equals("1")){
                    currency.setNumerical(currency.getNumerical()+5);
                }else if (order.getBuyType().equals("2")){
                    currency.setNumerical(currency.getNumerical()+10);
                }else if (order.getBuyType().equals("3")){
                    currency.setNumerical(currency.getNumerical()+20);
                }else if (order.getBuyType().equals("4")){
                    currency.setNumerical(currency.getNumerical()+30);
                }else if (order.getBuyType().equals("5")){
                    currency.setNumerical(currency.getNumerical()+50);
                }else if (order.getBuyType().equals("6")){
                    currency.setNumerical(currency.getNumerical()+100);
                }else {
                    return;
                }
                paVipCurrencyService.updateById(currency);
                // 添加日志
                PaConsumption consumption = new PaConsumption();
                consumption.setAddOrSub("add");
                consumption.setUserid(order.getMemberId());
                consumption.setValue(currency.getNumerical());
                consumption.setCreatetime(new Date());
                paConsumptionService.save(consumption);
            }
        } catch (ValidationException e) {
            e.printStackTrace();
        }
    }

//    public PaOrder jsPay(PaOrder order, HttpServletRequest request) {
//        com.wechat.pay.java.service.payments.jsapi.model.PrepayRequest prepayRequest = new com.wechat.pay.java.service.payments.jsapi.model.PrepayRequest();
//        prepayRequest.setAppid(weChatConfig.getAppId());
//        prepayRequest.setMchid(weChatConfig.getMerchantId());
//        String tradeNo = WeChatUtil.generateTradeNumber();
//        prepayRequest.setOutTradeNo(tradeNo);
//        prepayRequest.setDescription("钻石购买");
//        prepayRequest.setNotifyUrl(weChatConfig.notifyUrl);
//        com.wechat.pay.java.service.payments.jsapi.model.Amount amount = new com.wechat.pay.java.service.payments.jsapi.model.Amount();
//        amount.setTotal((order.getAmount().multiply(new BigDecimal("100"))).intValue());
//        prepayRequest.setAmount(amount);
//        Payer payer = new Payer();
//        payer.setOpenid(order.getOpenid());
//        prepayRequest.setPayer(payer);
//        // 调用下单方法，得到应答
//        PrepayResponse response;
//        try {
//            com.wechat.pay.java.service.payments.jsapi.model.PrepayResponse prepay = jsapiService.prepay(prepayRequest);
//            //预支付成功，创建预支付订单
//            order.setOrderNum(tradeNo);
//            order.setPayStatus(PayState.TO_BE_PAID);
//            order.setCreateTime(new Date());
//            order.setState("t");
//            order.setH5Url(null);
//            paOrderService.save(order);
//            return order;
//        } catch (HttpException e) { // 发送HTTP请求失败
//            e.printStackTrace();
//        } catch (ServiceException e) { // 服务返回状态小于200或大于等于300，例如500
//            e.printStackTrace();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return null;
//    }
}
