package com.pixelai.api.log.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.annotation.SysLog;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.log.entity.AcVisits;
import com.pixelai.api.log.service.AcVisitsService;
import com.pixelai.api.log.vo.AcVisitsVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author mzl
 * @since 2020-07-09
 */
@Api(tags = "访问记录")
@RestController
@RequestMapping("/acVisits")
public class AcVisitsController extends BaseController {

    private static final Logger          LOG = LoggerFactory.getLogger(AcVisitsController.class);
    @Autowired
    private              AcVisitsService entityService;

    @SysLog(action = "acVisits:findByPage", value = "系统日志：页面访问记录列表")
    @ApiOperation(value = "分页查询访问记录", notes = "分页查询访问记录")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<AcVisitsVo>> findByPage(final AcVisitsVo example, final Page page) {
        IPage<AcVisitsVo> records = entityService.findByPage(example, page);
        return new Result(true, records);
    }

    @SysLog(action = "acVisits:delete", value = "系统日志：删除页面访问记录")
    @ApiOperation(value = "删除访问记录")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "访问记录ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        entityService.removeById(id);
        return new Result(true, "成功删除访问记录", null);
    }

    @SysLog(action = "acVisits:one", value = "系统日志：查询页面访问记录详细")
    @ApiOperation(value = "查询单个访问记录")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "访问记录ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        AcVisits entity = entityService.getById(id);
        return new Result(true, entity);
    }

    @SysLog(action = "acVisits:add", value = "系统日志：添加页面访问记录")
    @ApiOperation(value = "添加访问记录")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final AcVisits entity) {
        entityService.save(entity);
        return new Result(true, "成功保存访问记录", null);
    }

    @SysLog(action = "acVisits:update", value = "系统日志：修改修改访问记录")
    @ApiOperation(value = "修改访问记录")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final AcVisits entity) {
        if (null == entity.getId()) {
            throw new RuntimeException("id不能为空");
        }
        entityService.updateById(entity);
        return new Result(true, "成功修改访问记录", null);
    }

    @SysLog(action = "acVisits:all", value = "系统日志：全部页面访问记录")
    @ApiOperation(value = "全部访问记录")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<AcVisitsVo>> all(AcVisitsVo example) {
        List<AcVisitsVo> entitys = entityService.findByExample(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}

