package com.pixelai.api.pa.controller;

import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.vo.PaCollectionVo;
import com.pixelai.api.user.service.AcUserService;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaCollection;
import com.pixelai.api.pa.service.PaCollectionService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-12-05
 */
@Api(tags = "用户收藏")
@RestController
@RequestMapping("/paCollection")
public class PaCollectionController extends BaseController {

    private static final Logger       LOG = LoggerFactory.getLogger(PaCollectionController.class);
    @Autowired
    private              PaCollectionService entityService;
    @Autowired
    private AcUserService acUserService;

    @SysLog(action = "findByPage", value = "分页查询收藏")
    @ApiOperation(value = "分页查询收藏", notes = "分页查询收藏")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<PaCollectionVo>> findByPage(final PaCollection example, final Page page) {
        IPage<PaCollectionVo>        records    = entityService.findByPage(example,page);
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除收藏")
    @ApiOperation(value = "删除收藏")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "收藏ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        try {
            PaCollection paCollection = entityService.getById(id);
            paCollection.setState("f");
            entityService.updateById(paCollection);
            return new Result(true, "成功删除收藏", null);
        } catch (DataIntegrityViolationException e) {
            LOG.error("删除收藏失败", e);
            return new Result(false, "删除收藏失败", "该收藏不能删除，存在其他关联数据");
        } catch (Exception e) {
            LOG.error("删除收藏失败", e);
            return new Result(false, "删除收藏失败", e.getMessage());
        }
    }

//    @SysLog(action = "one", value = "查询单个收藏")
//    @ApiOperation(value = "查询单个收藏")
//    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
//    @ApiImplicitParam(name = "id", value = "收藏ID", required = true, paramType = "path")
//    public Result one(@PathVariable final Integer id) {
//        try {
//            PaCollection entity = entityService.getById(id);
//            return new Result(true, entity);
//        } catch (Exception e) {
//            LOG.error("查询单个收藏失败", e);
//            return new Result(false, new PaCollection());
//        }
//    }

    @SysLog(action = "add", value = "收藏作品")
    @ApiOperation(value = "收藏作品")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final PaCollection entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "保存收藏失败", errorMsg, map);
            } else {
                Integer userid=acUserService.getUserId();
                if (userid==null){
                    return new Result(false, "用户不存在");
                }
                Integer count=entityService.lambdaQuery()
                        .eq(PaCollection::getUserId,userid)
                        .eq(PaCollection::getCreationId,entity.getCreationId())
                        .eq(PaCollection::getState,"t")
                        .count();
                if(count>0){
                    return new Result(false, "请勿重复收藏作品");
                }
                entity.setState("t");
                entity.setUserId(userid);
                entity.setCreatetime(new Date());
                entityService.save(entity);
                return new Result(true, "成功保存收藏", null);
            }
        } catch (Exception e) {
            LOG.error("添加收藏失败", e);
            return new Result(false, "保存收藏失败", e.getMessage());
        }
    }

//    @SysLog(action = "update", value = "修改收藏")
//    @ApiOperation(value = "修改收藏")
//    @RequestMapping(method = {RequestMethod.PUT})
//    public Result update(@Valid @RequestBody final PaCollection entity, final BindingResult result) {
//        try {
//            if (result.hasErrors()) {
//                Map<String, String> map      = this.getErrors(result);
//                String              errorMsg = map.entrySet().iterator().next().getValue();
//                return new Result(false, "修改收藏失败", errorMsg, map);
//            } else {
//                if (null == entity.getId()) {
//                    throw new RuntimeException("id不能为空");
//                }
//                entityService.updateById(entity);
//                return new Result(true, "成功修改收藏", null);
//            }
//        } catch (Exception e) {
//            LOG.error("修改收藏失败", e);
//            return new Result(false, "修改收藏失败", e.getMessage());
//        }
//    }
//
//    @ApiOperation(value = "全部收藏")
//    @RequestMapping(value = "all", method = RequestMethod.GET)
//    public Result<List<PaCollection>> all(PaCollection example) {
//        List<PaCollection> entitys = entityService.list(example);
//        if (null != entitys) {
//            return new Result<>(true, entitys);
//        }
//        return new Result<>(true, Collections.emptyList());
//    }

}
