package com.pixelai.api.pa.controller;

import com.base.annotation.Unsecured;
import com.pixelai.api.component.dao.CpAttachmentMapper;
import com.pixelai.api.component.entity.CpAttachment;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.service.CpPhotoService;
import com.pixelai.api.pa.dao.PaCreationMapper;
import com.pixelai.api.pa.dao.PaCreationReleaseMapper;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.entity.PaRecommend;
import com.pixelai.api.pa.entity.PaServices;
import com.pixelai.api.pa.service.PaCreationService;
import com.pixelai.api.pa.service.PaRecommendService;
import com.pixelai.api.pa.service.PaServicesService;
import com.pixelai.api.pa.service.impl.PaCreationReleaseServiceImpl;
import com.pixelai.api.pa.vo.PaCreationReleaseVo;
import com.pixelai.api.pa.vo.PaCreationVo;
import com.pixelai.api.picture.vo.AiPictureVo;
import com.pixelai.api.user.service.AcUserService;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaCreationRelease;
import com.pixelai.api.pa.service.PaCreationReleaseService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-12-05
 */
@Api(tags = "作品发布")
@RestController
@RequestMapping("/paCreationRelease")
public class PaCreationReleaseController extends BaseController {

    private static final Logger       LOG = LoggerFactory.getLogger(PaCreationReleaseController.class);
    @Autowired
    private              PaCreationReleaseService entityService;
    @Autowired
    private PaCreationReleaseMapper paCreationReleaseMapper;
    @Autowired
    private AcUserService acUserService;
    @Autowired
    private PaRecommendService paRecommendService;
    @Autowired
    private PaCreationService paCreationService;
    @Autowired
    private PaServicesService paServicesService;


    @SysLog(action = "findByPage", value = "分页查询作品")
    @ApiOperation(value = "分页查询作品", notes = "分页查询作品")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<PaCreationReleaseVo>> findByPage(final PaCreationReleaseVo example, final Page page) {
        IPage<PaCreationReleaseVo>        records    = entityService.findByPage(example,page);
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除作品")
    @ApiOperation(value = "删除作品")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "作品ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        try {
            PaCreationRelease paCreationRelease = entityService.getById(id);
            paCreationRelease.setState("f");
            entityService.updateById(paCreationRelease);
//            entityService.removeById(id);
            return new Result(true, "成功删除作品", null);
        } catch (DataIntegrityViolationException e) {
            LOG.error("删除作品失败", e);
            return new Result(false, "删除作品失败", "该作品不能删除，存在其他关联数据");
        } catch (Exception e) {
            LOG.error("删除作品失败", e);
            return new Result(false, "删除作品失败", e.getMessage());
        }
    }

    @SysLog(action = "one", value = "查询单个作品")
    @ApiOperation(value = "查询单个作品")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "作品ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        try {
            PaCreationReleaseVo entity = paCreationReleaseMapper.findById(id);
            return new Result(true, entity);
        } catch (Exception e) {
            LOG.error("查询单个作品失败", e);
            return new Result(false, new PaCreationRelease());
        }
    }

    @SysLog(action = "add", value = "添加共享作品")
    @ApiOperation(value = "添加共享作品")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final PaCreationRelease entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "保存作品失败", errorMsg, map);
            } else {
//                if(entity.getCreationId()==null){
//                    return new Result(false, "", "作品id为空");
//                }
                Integer count=entityService.lambdaQuery()
                        .eq(PaCreationRelease::getCreationId, entity.getCreationId())
                        .eq(PaCreationRelease::getState,"t")
                        .count();
                if(count>0){
                    return new Result(false, "", "请勿重复共享作品");
                }
                Integer userId=acUserService.getUserId();
                entity.setUserId(userId);
                entity.setCreatetime(new Date());
                entity.setState("t");
                entity.setPhotoSourceId(entity.getPhotoSourceId());
                entity.setPhotoTargetId(entity.getPhotoTargetId());
                entity.setReviewStatus(1);
                entityService.save(entity);
                return new Result(true, "成功保存作品", null);
            }
        } catch (Exception e) {
            LOG.error("添加作品失败", e);
            return new Result(false, "保存作品失败", e.getMessage());
        }
    }

    @SysLog(action = "reviewImages", value = "审核或推荐图片")
    @ApiOperation(value = "审核或推荐图片",notes = "审核或推荐图片")
    @RequestMapping(value="reviewImages",method = {RequestMethod.GET})
    public Result<String> reviewImages(@RequestParam("id") Integer id,@RequestParam("status") Integer status){
        try {
            if(!(status==1 || status==2 || status==3 || status==4 || status==5)){
                return new Result(false, "","审核失败，审核参数错误");
            }
            PaCreationRelease entity = entityService.getById(id);
            //推荐作品时，未处于通过或取消推荐状态不可以推荐
            if(status==4 && entity!=null  && !(entity.getReviewStatus()==2 ||entity.getReviewStatus()==5)){
                return new Result(false, "","审核失败，作品未审核或已推荐");
            }
            entity.setReviewer(acUserService.getUserId());
            entity.setReviewStatus(status);
            entityService.updateById(entity);
            //如果推荐则加入推荐表，取消推荐则在推荐表删除
            if(entity.getReviewStatus()==4){
                entity=entityService.getById(entity.getId());
                PaCreation paCreation=paCreationService.getById(entity.getCreationId());
                PaServices paServices=paServicesService.getById(paCreation.getServiceId());
                PaRecommend paRecommend=new PaRecommend();
                paRecommend.setNumber(0);
                paRecommend.setCreationId(entity.getCreationId());
                paRecommend.setUserId(acUserService.getUserId());
                paRecommend.setCreatetime(new Date());
                paRecommend.setState("t");
                paRecommend.setPhotoSourceId(entity.getPhotoSourceId());
                paRecommend.setPhotoTargetId(entity.getPhotoTargetId());
                paRecommend.setServiceId(paServices.getId());
                paRecommendService.save(paRecommend);
                entity.setRecommendId(paRecommend.getId());
                entityService.updateById(entity);
            }else if(entity.getReviewStatus()==5){
                paRecommendService.lambdaUpdate()
                        .set(PaRecommend::getState,"f")
                        .eq(PaRecommend::getId,entity.getRecommendId()).update();
            }
            return new Result(true, entity);
        } catch (Exception e) {
            LOG.error("审核单个作品失败", e);
            return new Result(false, new PaCreationRelease());
        }
    }


//
//    @SysLog(action = "update", value = "修改作品")
//    @ApiOperation(value = "修改作品")
//    @RequestMapping(method = {RequestMethod.PUT})
//    public Result update(@Valid @RequestBody final PaCreationRelease entity, final BindingResult result) {
//        try {
//            if (result.hasErrors()) {
//                Map<String, String> map      = this.getErrors(result);
//                String              errorMsg = map.entrySet().iterator().next().getValue();
//                return new Result(false, "修改作品失败", errorMsg, map);
//            } else {
//                if (null == entity.getId()) {
//                    throw new RuntimeException("id不能为空");
//                }
//                entityService.updateById(entity);
//                return new Result(true, "成功修改作品", null);
//            }
//        } catch (Exception e) {
//            LOG.error("修改作品失败", e);
//            return new Result(false, "修改作品失败", e.getMessage());
//        }
//    }
//
//    @ApiOperation(value = "全部作品")
//    @RequestMapping(value = "all", method = RequestMethod.GET)
//    public Result<List<PaCreationRelease>> all(PaCreationRelease example) {
//        List<PaCreationRelease> entitys = entityService.list(example);
//        if (null != entitys) {
//            return new Result<>(true, entitys);
//        }
//        return new Result<>(true, Collections.emptyList());
//    }

}
