package com.pixelai.api.pa.controller;

import com.aliyun.core.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.base.annotation.SysLog;
import com.base.annotation.Unsecured;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.domain.apy.OrderPayAppService;
import com.pixelai.api.pa.entity.PaOrder;
import com.pixelai.api.pa.service.PaOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.Collections;
import java.util.Map;
import java.util.List;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2025-01-19
 */
@Api(tags = "pa订单管理")
@RestController
@RequestMapping("/paOrder")
public class PaOrderController extends BaseController {

        private static final Logger       LOG = LoggerFactory.getLogger(PaOrderController.class);
        @Autowired
        private PaOrderService entityService;
        @Autowired
        private OrderPayAppService payAppService;

        @SysLog(action = "findByPage", value = "分页查询订单")
        @ApiOperation(value = "分页查询订单", notes = "分页查询订单")
        @RequestMapping(method = RequestMethod.GET)
        @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
                @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
        public Result<IPage<PaOrder>> findByPage(final PaOrder example, final Page page) {
            IPage<PaOrder>        records    = entityService.page(example,page);
            return new Result(true, records);
        }

        @SysLog(action = "one", value = "查询单个订单")
        @ApiOperation(value = "查询单个订单")
        @RequestMapping(value = "{id}", method = {RequestMethod.GET})
        @ApiImplicitParam(name = "id", value = "订单ID", required = true, paramType = "path")
        public Result one(@PathVariable final Integer id) {
            try {
                PaOrder entity = entityService.getById(id);
                return new Result(true, entity);
            } catch (Exception e) {
                LOG.error("查询单个订单失败", e);
                return new Result(false, new PaOrder());
            }
        }

        @SysLog(action = "add", value = "添加订单")
        @ApiOperation(value = "添加订单")
        @RequestMapping(method = {RequestMethod.POST})
        public Result add(@Valid @RequestBody final PaOrder entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "保存订单失败", errorMsg, map);
                } else {
                    entityService.save(entity);
                    return new Result(true, "成功保存订单", null);
                }
            } catch (Exception e) {
                LOG.error("添加订单失败", e);
                return new Result(false, "保存订单失败", e.getMessage());
            }
        }


        @ApiOperation(value = "全部订单")
        @RequestMapping(value = "all", method = RequestMethod.GET)
        public Result<List<PaOrder>> all(PaOrder example) {
            List<PaOrder> entitys = entityService.list(example);
            if (null != entitys) {
                return new Result<>(true, entitys);
            }
            return new Result<>(true, Collections.emptyList());
        }

    @ApiOperation(value = "h5支付")
    @RequestMapping(value = "h5Pay", method = RequestMethod.GET)
    public Result h5Pay(PaOrder order,HttpServletRequest request) {
        // 调用业务逻辑获取支付 URL
        PaOrder order1 = payAppService.h5Pay(order,request);
        // 如果没有获取到支付链接，返回失败的结果
        if (StringUtils.isBlank(order1.getH5Url())) {
            return new Result<>(false, "支付失败");
        }
        // 返回成功的结果
        return new Result<>(true, order1);
    }

//    @ApiOperation(value = "js支付")
//    @RequestMapping("jsPay")
//    public Result jsPay(PaOrder order, HttpServletRequest request) {
//        PaOrder order1 = payAppService.jsPay(order, request);
//        if (Objects.isNull(order1)) {
//            return new Result<>(false, "支付失败");
//        }
//        return new Result<>(true, order1);
//    }

    @ApiOperation(value = "h5回调")
    @PostMapping(value = "payNotify")
    @Unsecured
    public void payNotify(HttpServletRequest request) {
        try {
            payAppService.payNotify(request);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}

