package com.pixelai.api.pa.controller;

import com.base.annotation.Unsecured;
import com.pixelai.api.component.entity.CpAttachment;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.pa.dao.PaPictureWallMapper;
import com.pixelai.api.pa.service.PaVipCurrencyService;
import com.pixelai.api.pa.vo.PaPictureWallVo;
import com.pixelai.api.picture.vo.AiPictureVo;
import com.pixelai.api.user.service.AcUserService;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaPictureWall;
import com.pixelai.api.pa.service.PaPictureWallService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2025-01-16
 */
@Api(tags = "作品墙")
@RestController
@RequestMapping("/paPictureWall")
public class PaPictureWallController extends BaseController {

    private static final Logger       LOG = LoggerFactory.getLogger(PaPictureWallController.class);
    @Autowired
    private              PaPictureWallService entityService;
    @Autowired
    private AcUserService acUserService;
    @Autowired
    private PaPictureWallMapper paPictureWallMapper;
    @Autowired
    private PaVipCurrencyService paVipCurrencyService;

    @SysLog(action = "findByPage", value = "分页查询作品墙")
    @ApiOperation(value = "分页查询作品墙", notes = "分页查询作品墙")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<PaPictureWallVo>> findByPage(final PaPictureWallVo example, final Page page) {
        IPage<PaPictureWallVo>        records    = entityService.findByPage(example,page);
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除作品墙")
    @ApiOperation(value = "删除作品墙")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "作品墙ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        try {
            entityService.lambdaUpdate().set(PaPictureWall::getState,"f")
                    .eq(PaPictureWall::getId,id).update();
            return new Result(true, "成功删除作品墙", null);
        } catch (DataIntegrityViolationException e) {
            LOG.error("删除作品墙失败", e);
            return new Result(false, "删除作品墙失败", "该作品墙不能删除，存在其他关联数据");
        } catch (Exception e) {
            LOG.error("删除作品墙失败", e);
            return new Result(false, "删除作品墙失败", e.getMessage());
        }
    }

    @SysLog(action = "one", value = "查询单个作品墙")
    @ApiOperation(value = "查询单个作品墙")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "作品墙ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        try {
            PaPictureWall entity = entityService.lambdaQuery()
                    .eq(PaPictureWall::getId,id)
                    .eq(PaPictureWall::getState,"t").one();
            return new Result(true, entity);
        } catch (Exception e) {
            LOG.error("查询单个作品墙失败", e);
            return new Result(false, new PaPictureWall());
        }
    }

    @SysLog(action = "add", value = "添加作品墙")
    @ApiOperation(value = "添加作品墙")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final PaPictureWall entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "保存作品墙失败", errorMsg, map);
            } else {
                entity.setCreatetime(new Date());
                entity.setUserid(acUserService.getUserId());
                entity.setState("t");
                entityService.save(entity);
                return new Result(true, "成功保存作品墙", null);
            }
        } catch (Exception e) {
            LOG.error("添加作品墙失败", e);
            return new Result(false, "保存作品墙失败", e.getMessage());
        }
    }

    @SysLog(action = "update", value = "修改作品墙")
    @ApiOperation(value = "修改作品墙")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final PaPictureWall entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "修改作品墙失败", errorMsg, map);
            } else {
                if (null == entity.getId()) {
                    throw new RuntimeException("id不能为空");
                }
                entity.setCreatetime(new Date());
                entity.setUserid(acUserService.getUserId());
                entity.setState("t");
                entityService.updateById(entity);
                return new Result(true, "成功修改作品墙", null);
            }
        } catch (Exception e) {
            LOG.error("修改作品墙失败", e);
            return new Result(false, "修改作品墙失败", e.getMessage());
        }
    }

    @ApiOperation(value = "根据标签id查询推荐墙")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    @Unsecured
    public Result<List<PaPictureWallVo>> all(Integer labelId) {
        List<PaPictureWallVo> entitys = paPictureWallMapper.findById(labelId);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

    @SysLog(action = "PaPictureWall:geiOriginalPicture", value = "购买原图")
    @ApiOperation(value = "购买原图",notes = "购买原图")
    @RequestMapping(value="geiOriginalPicture",method = {RequestMethod.GET})
    public Result<PaPictureWallVo> geiOriginalPicture(Integer id)  {
        Integer userId=acUserService.getUserId();
        if(userId==null){
            return new Result(false, "用户未登录");
        }
        PaPictureWall paPictureWall=entityService.lambdaQuery()
                .eq(PaPictureWall::getId,id)
                .eq(PaPictureWall::getState,"t").one();
        if (paPictureWall ==null){
            return new Result(false, "作品不存在");
        }
        //进行扣费
        Boolean state=paVipCurrencyService.consumeOriginalPicture(userId,paPictureWall);
        if (state==false){
            return new Result(false, "余额不足");
        }
        return new Result(true, paPictureWall);
    }

}
