package com.pixelai.api.pa.controller;

import com.base.annotation.Unsecured;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.service.CpPhotoService;
import com.pixelai.api.pa.dao.PaCreationMapper;
import com.pixelai.api.pa.entity.PaServices;
import com.pixelai.api.pa.service.PaServicesService;
import com.pixelai.api.pa.vo.PaCreationReleaseVo;
import com.pixelai.api.pa.vo.PaCreationVo;
import com.pixelai.api.pa.vo.PaRecommendVo;
import com.pixelai.api.user.service.AcUserService;
import com.pixelai.utils.StringUtil;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaRecommend;
import com.pixelai.api.pa.service.PaRecommendService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-12-05
 */
@Api(tags = "作品推荐")
@RestController
@RequestMapping("/paRecommend")
public class PaRecommendController extends BaseController {

    private static final Logger       LOG = LoggerFactory.getLogger(PaRecommendController.class);
    @Autowired
    private              PaRecommendService entityService;
    @Autowired
    private CpPhotoService cpPhotoService;
    @Autowired
    private PaCreationMapper paCreationMapper;
    @Autowired
    private AcUserService acUserService;
    @Autowired
    private PaServicesService paServicesService;

    @Unsecured
    @SysLog(action = "findByPage", value = "分页查询推荐作品")
    @ApiOperation(value = "分页查询推荐作品", notes = "分页查询推荐作品")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<PaRecommend>> findByPage(final PaRecommendVo example, final Page page) {
        IPage<PaRecommendVo>        records    = entityService.findByPage(example,page);
        for(PaRecommendVo paRecommendVo :records.getRecords()){
            if(paRecommendVo.getCreationId()!=null){
                PaCreationVo paCreationVo=paCreationMapper.findById(paRecommendVo.getCreationId());
                paRecommendVo.setPath(paCreationVo.getPath());
            }
            if(paRecommendVo.getPhotoSourceId()!=null){
                CpPhoto cpPhoto=cpPhotoService.getById(paRecommendVo.getPhotoSourceId());
                paRecommendVo.setSourcePath(cpPhoto.getPath());
            }
            if(paRecommendVo.getPhotoTargetId()!=null){
                CpPhoto cpPhoto=cpPhotoService.getById(paRecommendVo.getPhotoTargetId());
                paRecommendVo.setPath(cpPhoto.getPath());
            }
            if(StringUtil.isBlank(paRecommendVo.getServiceName())){
                PaServices paServices=paServicesService.getById(paRecommendVo.getServiceId());
                paRecommendVo.setServiceName(paServices.getName());
            }
        }
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除推荐作品")
    @ApiOperation(value = "删除推荐作品")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "推荐作品ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        try {
            entityService.removeById(id);
            return new Result(true, "成功删除推荐作品", null);
        } catch (DataIntegrityViolationException e) {
            LOG.error("删除推荐作品失败", e);
            return new Result(false, "删除推荐作品失败", "该推荐作品不能删除，存在其他关联数据");
        } catch (Exception e) {
            LOG.error("删除推荐作品失败", e);
            return new Result(false, "删除推荐作品失败", e.getMessage());
        }
    }

    @SysLog(action = "one", value = "查询单个推荐作品")
    @ApiOperation(value = "查询单个推荐作品")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "推荐作品ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        try {
            PaRecommend entity = entityService.getById(id);
            return new Result(true, entity);
        } catch (Exception e) {
            LOG.error("查询单个推荐作品失败", e);
            return new Result(false, new PaRecommend());
        }
    }

    @SysLog(action = "add", value = "添加推荐作品")
    @ApiOperation(value = "添加推荐作品")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final PaRecommend entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "保存推荐作品失败", errorMsg, map);
            } else {
                entity.setUserId(acUserService.getUserId());
                entity.setState("t");
                entity.setCreatetime(new Date());
                entityService.save(entity);
                return new Result(true, "成功保存推荐作品", null);
            }
        } catch (Exception e) {
            LOG.error("添加推荐作品失败", e);
            return new Result(false, "保存推荐作品失败", e.getMessage());
        }
    }

    @SysLog(action = "update", value = "修改推荐作品")
    @ApiOperation(value = "修改推荐作品")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final PaRecommend entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "修改推荐作品失败", errorMsg, map);
            } else {
                if (null == entity.getId()) {
                    throw new RuntimeException("id不能为空");
                }
                entityService.updateById(entity);
                return new Result(true, "成功修改推荐作品", null);
            }
        } catch (Exception e) {
            LOG.error("修改推荐作品失败", e);
            return new Result(false, "修改推荐作品失败", e.getMessage());
        }
    }

    @ApiOperation(value = "全部推荐作品")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<PaRecommend>> all(PaRecommend example) {
        List<PaRecommend> entitys = entityService.list(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}
