package com.pixelai.api.pa.controller;

import com.base.annotation.Unsecured;
import com.pixelai.api.pa.dao.PaServiceOptionMapper;
import com.pixelai.api.pa.dao.PaServicesMapper;
import com.pixelai.api.pa.entity.PaServiceOption;
import com.pixelai.api.pa.service.PaServiceOptionService;
import com.pixelai.api.pa.vo.PaServiceOptionVo;
import com.pixelai.api.pa.vo.PaServiceVo;
import com.pixelai.utils.BeanCopyUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaServices;
import com.pixelai.api.pa.service.PaServicesService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-12-04
 */
@Api(tags = "应用服务")
@RestController
@RequestMapping("/paServices")
public class PaServicesController extends BaseController {

    private static final Logger       LOG = LoggerFactory.getLogger(PaServicesController.class);
    @Autowired
    private              PaServicesService entityService;
    @Autowired
    private PaServicesMapper paServicesMapper;
    @Autowired
    private PaServiceOptionService paServiceOptionService;
    @Autowired
    private PaServiceOptionMapper paServiceOptionMapper;

    @SysLog(action = "findByPage", value = "分页查询应用")
    @ApiOperation(value = "分页查询应用", notes = "分页查询应用")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<PaServices>> findByPage(final PaServices example, final Page page) {
        IPage<PaServices>        records    = entityService.page(example,page);
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除应用")
    @ApiOperation(value = "删除应用")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "应用ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        try {
            PaServices paServices=entityService.lambdaQuery().eq(PaServices::getId,id).one();
            paServices.setState("f");
            entityService.updateById(paServices);
            return new Result(true, "成功删除应用", null);
        } catch (DataIntegrityViolationException e) {
            LOG.error("删除应用失败", e);
            return new Result(false, "删除应用失败", "该应用不能删除，存在其他关联数据");
        } catch (Exception e) {
            LOG.error("删除应用失败", e);
            return new Result(false, "删除应用失败", e.getMessage());
        }
    }

    @SysLog(action = "one", value = "查询单个应用")
    @ApiOperation(value = "查询单个应用")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "应用ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        try {
            PaServices entity=entityService.getServiceById(id);
            PaServiceVo paServiceVo=new PaServiceVo();
            BeanCopyUtils.copyPropertiesIgnoreNull(entity,paServiceVo);
            List<PaServiceOptionVo> options=paServiceOptionMapper.findOptionsById(paServiceVo.getId());
            paServiceVo.setOptions(options);
            return new Result(true, paServiceVo);
        } catch (Exception e) {
            LOG.error("查询单个应用失败", e);
            return new Result(false, new PaServices());
        }
    }

    @SysLog(action = "add", value = "添加应用")
    @ApiOperation(value = "添加应用")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final PaServices entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "保存应用失败", errorMsg, map);
            } else {
                entity.setCreatetime(new Date());
                entity.setState("t");
                entityService.save(entity);
                return new Result(true, "成功保存应用", null);
            }
        } catch (Exception e) {
            LOG.error("添加应用失败", e);
            return new Result(false, "保存应用失败", e.getMessage());
        }
    }

    @SysLog(action = "update", value = "修改应用")
    @ApiOperation(value = "修改应用")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final PaServices entity, final BindingResult result) {
        try {
            if (result.hasErrors()) {
                Map<String, String> map      = this.getErrors(result);
                String              errorMsg = map.entrySet().iterator().next().getValue();
                return new Result(false, "修改应用失败", errorMsg, map);
            } else {
                if (null == entity.getId()) {
                    throw new RuntimeException("id不能为空");
                }
                entityService.updateById(entity);
                return new Result(true, "成功修改应用", null);
            }
        } catch (Exception e) {
            LOG.error("修改应用失败", e);
            return new Result(false, "修改应用失败", e.getMessage());
        }
    }

    @SysLog(action = "homeViews", value = "首页访问")
    @ApiOperation(value = "全部应用")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    @Unsecured
    public Result<List<PaServiceVo>> all() {
        List<PaServiceVo> entitys = paServicesMapper.findAllService();
        if (null != entitys) {
            for (PaServiceVo entity : entitys) {
//                if (entity.getType().equals("2")){
                    List<PaServiceOptionVo> options=paServiceOptionMapper.findOptionsById(entity.getId());
                    entity.setOptions(options);
//                }
            }
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}

