package com.pixelai.api.picture.schedule;

import com.alibaba.fastjson.JSONArray;
import com.base.helper.StringUtils;
import com.pixelai.api.component.entity.CpAttachment;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.service.CpAttachmentService;
import com.pixelai.api.component.service.CpPhotoService;
import com.pixelai.api.pa.entity.PaAiTryon;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.entity.PaServices;
import com.pixelai.api.pa.entity.PaVipCurrency;
import com.pixelai.api.pa.service.PaAiTryonService;
import com.pixelai.api.pa.service.PaCreationService;
import com.pixelai.api.pa.service.PaServicesService;
import com.pixelai.api.pa.service.PaVipCurrencyService;
import com.pixelai.api.picture.service.AiPictureService;
import com.pixelai.api.picture.util.AliyunOssUtil;
import com.pixelai.api.picture.vo.ResultVo;
import com.volcengine.service.visual.IVisualService;
import com.volcengine.service.visual.impl.VisualServiceImpl;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

@Configuration
@EnableScheduling
public class ScheduledTasks {
    @Value("${aliyun.aliyunKey}")
    private String API_KEY;
    @Autowired
    RestTemplate restTemplate;
    @Autowired
    private PaCreationService paCreationService;
    @Autowired
    private PaServicesService paServicesService;
    @Autowired
    private PaVipCurrencyService paVipCurrencyService;
//    @Autowired
//    private PaAiTryonService paAiTryonService;
    @Autowired
    private CpAttachmentService cpAttachmentService;
    @Autowired
    private AiPictureService aiPictureService;
    @Value("${volcengine.AccessKeyID}")
    private String volcengineKEY;
    @Value("${volcengine.SecretAccessKey}")
    private String volcengineSecret;
    @Value("${picture.fail}")
    private String fail;



    //每隔30秒拉取一次阿里云、火山生成结果
    @Scheduled(fixedRate = 30*1000)
    public void getCreationUrl() throws Exception {
        //2阿里云，3火山
        List<PaCreation> paCreations= paCreationService.lambdaQuery()
                .eq(PaCreation::getState,"create")
                .in(PaCreation::getManufacturers, Arrays.asList(2,3)).list();
        for (PaCreation paCreation : paCreations){
            try {
                if (paCreation.getManufacturers()==2) {
                    //如果不存在任务id则生成失败，并返回钻石
                    if (StringUtils.isEmpty(paCreation.getTaskId())) {
                        //更换失败图片
                        updateUrlByPhotoId(paCreation, fail, "fail","无任务id");
                        //退费
                        PaServices paServices = paServicesService.lambdaQuery().eq(PaServices::getId, paCreation.getServiceId()).one();
                        PaVipCurrency paVipCurrency = paVipCurrencyService.lambdaQuery().eq(PaVipCurrency::getUserid, paCreation.getUserId()).one();
                        paVipCurrency.setNumerical(paVipCurrency.getNumerical() + paServices.getPrice());
                        paVipCurrencyService.updateById(paVipCurrency);
                        continue;
                    }
                    if (paCreation.getServiceId() == 4 ||paCreation.getServiceId() == 6 || paCreation.getServiceId() == 7) {
                        ResultVo resultVo = getImageByTaskId(paCreation.getTaskId());
                        if (StringUtils.isNotEmpty(resultVo.getStatus()) && resultVo.getStatus().equals("SUCCEEDED")) {
                            updateUrlByPhotoId(paCreation, resultVo.getUrl(), "t","");
                        }else if (StringUtils.isNotEmpty(resultVo.getStatus()) && resultVo.getStatus().equals("FAILED")){
                            updateUrlByPhotoId(paCreation, fail, "fail", resultVo.getReason());
                        }
                        continue;
                    }
                    //换衣
                    if (paCreation.getServiceId() == 5) {
                        ResultVo resultVo = getImageByTaskId(paCreation.getTaskId());
                        if (StringUtils.isNotEmpty(resultVo.getStatus()) && resultVo.getStatus().equals("SUCCEEDED")) {
                            //更新用户表
                            String ossUrl = updateUrlByPhotoId(paCreation, resultVo.getUrl(), "t","");
                            //根据作品id更新预览表
//                            paAiTryonService.lambdaUpdate().set(PaAiTryon::getTargetUrl, ossUrl).eq(PaAiTryon::getCreationId, paCreation.getId()).update();
                        }else if (StringUtils.isNotEmpty(resultVo.getStatus()) && resultVo.getStatus().equals("FAILED")){
                            updateUrlByPhotoId(paCreation, fail, "fail", resultVo.getReason());
                            //删除预览表内容
//                            paAiTryonService.lambdaUpdate().eq(PaAiTryon::getCreationId, paCreation.getId()).remove();
                        }
                    }
                }else if (paCreation.getManufacturers()==3) {
                    if (paCreation.getServiceId() == 8) {
                        ResultVo resultVo = cVSync2AsyncGetResult(paCreation,"high_aes_general_v21_L");
                        if (StringUtils.isNotEmpty(resultVo.getStatus()) && resultVo.getStatus().equals("Success")) {
                            updateUrlByPhotoId(paCreation, resultVo.getUrl(), "t","");
                        }else if (StringUtils.isNotEmpty(resultVo.getStatus()) && resultVo.getStatus().equals("fail")){
                            updateUrlByPhotoId(paCreation, fail, "fail",resultVo.getReason());
                        }
                    }
                }
            }catch (Exception e){
                aiPictureService.failureHandling(paCreation,e);
            }

        }
    }

    public ResultVo cVSync2AsyncGetResult(PaCreation paCreation,String reqKey) throws Exception {
        String taskId=paCreation.getTaskId();
        IVisualService visualService = VisualServiceImpl.getInstance();
        visualService.setAccessKey(volcengineKEY);
        visualService.setSecretKey(volcengineSecret);
        JSONObject req=new JSONObject();
        req.put("req_key", reqKey);
        req.put("task_id", taskId);
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("return_url", true);
        JSONObject req_json = new JSONObject(map);
        req.put("req_json", req_json.toString());

        try {
            ResultVo resultVo=new ResultVo();
            JSONObject response = (JSONObject)visualService.cvSync2AsyncGetResult(req);
            String status=response.getString("message");
            if (status.equals("Success")){
                JSONObject data=response.getJSONObject("data");
                JSONArray imageUrls=data.getJSONArray("image_urls");
                if (imageUrls!=null){
                    String imageUrl=imageUrls.get(0).toString();
                    resultVo.setUrl(imageUrl);
                    resultVo.setStatus(status);
                }
            }
            return resultVo;
        } catch (Exception e) {
            aiPictureService.failureHandling(paCreation,e);
            ResultVo resultVo=new ResultVo();
            resultVo.setStatus("fail");
            resultVo.setReason(paCreation.getReason());
            return resultVo;
        }
    }


    public String updateUrlByPhotoId(PaCreation paCreation,String url,String state,String reason) throws IOException {
        MultipartFile file=AliyunOssUtil.fileUrlConvertToMultipartFile(url);
        String ossUrl=AliyunOssUtil.uploadFile(file);
        //更新图片url,AI换衣特殊处理
//        if (paCreation.getServiceId()==5){
//            cpAttachmentService.lambdaUpdate()
//                    .set(CpAttachment::getCpLabelId,5)
//                    .set(CpAttachment::getPath,ossUrl)
//                    .set(CpAttachment::getState,state)
//                    .eq(CpAttachment::getId,paCreation.getPhotoId()).update();
//        }else{
            cpAttachmentService.lambdaUpdate()
                    .set(CpAttachment::getPath,ossUrl)
                    .set(CpAttachment::getState,"t")//图片状态永远为t
                    .eq(CpAttachment::getId,paCreation.getPhotoId()).update();
//        }
        //更新作品状态
        paCreationService.lambdaUpdate()
                .set(PaCreation::getState,state)//作品状态为create、t、fail
//                .set(PaCreation::getReason,reason)
                .eq(PaCreation::getId,paCreation.getId()).update();
        return ossUrl;
    }

    //阿里云异步获取结果
    public ResultVo getImageByTaskId(String taskId) throws Exception{
        ResultVo resultVo =new ResultVo();
        try {
            HttpHeaders h = new HttpHeaders();
            h.set("Authorization", API_KEY);

            HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
            ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                    "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                    HttpMethod.GET,
                    requestGetFromTask,
                    Map.class,
                    taskId
            );
            Map NewResponse = responseGetFromTask.getBody();
            String status=(String)((Map) NewResponse.get("output")).get("task_status");
            if ("SUCCEEDED".equals(status)) {//检查任务状态
                List<Map<String, String>> results=(List<Map<String, String>>) ((Map) NewResponse.get("output")).get("results");
                if(results==null){
                    resultVo.setStatus(status);
                    resultVo.setUrl((String) ((Map) NewResponse.get("output")).get("image_url"));
                }else{
                    resultVo.setStatus(status);
                    resultVo.setUrl(results.get(0).get("url"));//获取返回的阿里云云上的图片url
                }
            }else if("FAILED".equals(status)){
                resultVo.setStatus(status);
                resultVo.setReason((String)((Map) NewResponse.get("output")).get("message"));
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return resultVo;
    }
}
