package com.pixelai.api.picture.service.impl;

import com.alibaba.dashscope.aigc.imagesynthesis.ImageSynthesis;
import com.alibaba.dashscope.aigc.imagesynthesis.ImageSynthesisParam;
import com.alibaba.dashscope.aigc.imagesynthesis.ImageSynthesisResult;
import com.alibaba.dashscope.exception.ApiException;
import com.alibaba.dashscope.exception.NoApiKeyException;
import com.alibaba.dashscope.utils.JsonUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.base.annotation.SysLog;
import com.pixelai.api.component.entity.CpAttachment;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.service.CpAttachmentService;
import com.pixelai.api.component.service.CpPhotoService;
import com.pixelai.api.pa.entity.PaAiTryon;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.service.PaAiTryonService;
import com.pixelai.api.pa.service.PaCreationService;
import com.pixelai.api.picture.config.VolcengineExConfig;
import com.pixelai.api.picture.service.AiPictureService;
import com.pixelai.api.picture.util.*;
import com.pixelai.api.picture.vo.AiPictureVo;
import com.pixelai.api.picture.vo.DirVo;
import com.pixelai.utils.StringUtil;
import com.volcengine.service.visual.model.request.VisualHumanSegmentRequest;
import com.volcengine.service.visual.model.request.VisualJPCartoonRequest;
import com.volcengine.service.visual.model.response.VisualHighAesSmartDrawingResponse;
import com.volcengine.service.visual.model.response.VisualHumanSegmentResponse;
import com.volcengine.service.visual.model.response.VisualJPCartoonResponse;
import okhttp3.*;
import okhttp3.MediaType;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.http.*;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;
import java.io.*;
import java.util.*;
import com.volcengine.service.visual.IVisualService;
import com.volcengine.service.visual.impl.VisualServiceImpl;
import com.volcengine.service.visual.model.request.VisualImg2Video3DRequest;
import com.volcengine.service.visual.model.response.VisualImg2Video3DResponse;

import java.util.ArrayList;

@Service
public class AiPictureServiceImpl implements AiPictureService {
    //指定生成图片路径
    @Value("${external.resource.dir}")
    private String externalResourceDir;
    @Value("${aliyun.aliyunKey}")
    private String API_KEY;
    @Value("${volcengine.AccessKeyID}")
    private String volcengineKEY;
    @Value("${volcengine.SecretAccessKey}")
    private String volcengineSecret;

    @Autowired
    private CpPhotoService cpPhotoService;
    @Autowired
    private CpAttachmentService cpAttachmentService;
    @Autowired
    private PaCreationService paCreationService;
    @Autowired
    private PaAiTryonService paAiTryonService;
    @Autowired
    private VolcengineExConfig volcengineExConfig;
    @Value("${picture.fail}")
    private String fail;
    @Autowired
    private RestTemplateBuilder restTemplateBuilder;
    @Bean
    public RestTemplate getRestTemplate() {
        return restTemplateBuilder.build();
    }
    @Autowired
    RestTemplate restTemplate;

    @Override
    @SysLog(action = "AiPicture:ImageColoring", value = "百度：黑白图片上色")
    @Async
    public void ImageColoring(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //请求百度URL
            String url="https://aip.baidubce.com/rest/2.0/image-process/v1/colourize";
            String base64=this.ActivateMagic(aiPictureVo.getImageUrl(),url,null);
            //保存base64为图片到oss
            MultipartFile img=AliyunOssUtil.base64ToMultipartFile(base64,System.currentTimeMillis()+"");
            String imgUrl=AliyunOssUtil.uploadFile(img);
            CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
            cpAttachment.setPath(imgUrl);
            cpAttachmentService.updateById(cpAttachment);
            paCreation.setState("t");
            paCreationService.updateById(paCreation);
//        return cpAttachment;
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    public void failureHandling(PaCreation paCreation,Exception e){
        CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
        cpAttachment.setPath(fail);
        cpAttachmentService.updateById(cpAttachment);
        String message=e.getMessage();
        if (StringUtil.isNotBlank(message) && JSONUtil.isJSON(message)){
            JSONObject jsonObject=JSONObject.parseObject(message);
            if (jsonObject!=null) {
                Integer code = Integer.valueOf(jsonObject.getString("code"));
                paCreation.setReason(VolcengineUtil.getErrorMessage(code));
            }
        }else if(StringUtil.isNotBlank(message) && message.equals("400 Bad Request")){
            paCreation.setReason("输入内容不符合规范");
        }
        paCreation.setSystemCause(message);
        paCreation.setState("fail");
        paCreationService.updateById(paCreation);
    }

    @Override
    @SysLog(action = "AiPicture:CharacterAnime", value = "百度：人物动漫化")
    @Async
    public void CharacterAnime(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //请求百度URL
            String url="https://aip.baidubce.com/rest/2.0/image-process/v1/selfie_anime";
            String base64=this.ActivateMagic(aiPictureVo.getImageUrl(),url,null);
            //保存base64为图片到oss
            MultipartFile img=AliyunOssUtil.base64ToMultipartFile(base64,System.currentTimeMillis()+"");
            String imgUrl=AliyunOssUtil.uploadFile(img);
            CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
            cpAttachment.setPath(imgUrl);
            cpAttachmentService.updateById(cpAttachment);
            paCreation.setState("t");
            paCreationService.updateById(paCreation);
//            return cpAttachment;
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    @Override
    @SysLog(action = "AiPicture:styleTrans", value = "百度：更换图片风格")
    @Async
    public void styleTrans(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //请求百度URL
            String url="https://aip.baidubce.com/rest/2.0/image-process/v1/style_trans";
            String base64=this.ActivateMagic(aiPictureVo.getImageUrl(),url,aiPictureVo.getOption());
            //保存base64为图片到oss
            MultipartFile img=AliyunOssUtil.base64ToMultipartFile(base64,System.currentTimeMillis()+"");
            String imgUrl=AliyunOssUtil.uploadFile(img);
            CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
            cpAttachment.setPath(imgUrl);
            cpAttachmentService.updateById(cpAttachment);
            paCreation.setState("t");
            paCreationService.updateById(paCreation);
//            return cpAttachment;
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    @Async
    @Override
    @SysLog(action = "AiPicture:textToImage", value = "阿里云：文字生成图片")
    public void textToImage(AiPictureVo aiPictureVo,PaCreation paCreation) throws Exception {
            //异步请求阿里云URL，获取任务id
            String taskId=((Map)this.text2Images(aiPictureVo,paCreation)).get("task_id").toString();
            paCreation.setTaskId(taskId);
            paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:textToImageByVolcengine", value = "火山引擎：文字生成图片")
    public void textToImageByVolcengine(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //请求火山API
            JSONObject req= new JSONObject();
            req.put("req_key", "high_aes_general_v21_L");
            req.put("prompt", aiPictureVo.getText());
            req.put("req_schedule_conf", aiPictureVo.getOption());//标准版：general_v20_9B_rephraser 美感版：general_v20_9B_pe
            req.put ("model_version", "general_v2.1_L");
            req.put ("return_url", true);
            req.put ("use_pre_llm", true);
            if (StringUtil.isNotBlank(aiPictureVo.getSize())){
                String str=aiPictureVo.getSize();
                String[] strlist=str.split("\\*");
                if (strlist.length==2){
                    req.put ("width",Integer.parseInt(strlist[0]) );
                    req.put ("height", Integer.parseInt(strlist[1]));
                }
            }

            JSONObject response =VolcengineUtil.CVSubmitTask(paCreation,req);
            if (response==null)
                return;
            JSONObject data=response.getJSONObject("data");
            String taskId=data.getString("task_id");
            //业务代码
            paCreation.setTaskId(taskId);
            paCreation.setState("create");
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

//    @Override
//    @Async
//    @SysLog(action = "AiPicture:aiTryonPreview", value = "阿里云：AI换衣预览模式")
//    public void aiTryonPreview(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
//        try {
//            PaAiTryon paAiTryon=paAiTryonService.lambdaQuery()
//                    .eq(PaAiTryon::getImageUrl,aiPictureVo.getImageUrl())
//                    .eq(PaAiTryon::getTopImageUrl,aiPictureVo.getTopImageUrl())
//                    .eq(StringUtil.isNotBlank(aiPictureVo.getBottomImageUrl()),PaAiTryon::getBottomImageUrl,aiPictureVo.getBottomImageUrl())
//                    .eq(StringUtil.isBlank(aiPictureVo.getBottomImageUrl()),PaAiTryon::getBottomImageUrl,null).one();
//
//            //如果不存在换衣记录，则请求阿里云获取并记录
//            if (paAiTryon==null){
//                //异步请求阿里云URL，获取任务id，tryon传参，上衣、下衣、人物
//                String taskId=((Map)this.tryon(aiPictureVo.getTopImageUrl(),aiPictureVo.getBottomImageUrl(),aiPictureVo.getImageUrl())).get("task_id").toString();
//                paCreation.setTaskId(taskId);
//                paCreationService.updateById(paCreation);
//                //保存换衣记录
//                paAiTryonService.save(new PaAiTryon(aiPictureVo.getImageUrl(),aiPictureVo.getTopImageUrl(),aiPictureVo.getBottomImageUrl(),null,paCreation.getId()));
//                return;
//            }
//            //如果存在换衣记录，则置换path，并标记生成成功
//            cpAttachmentService.lambdaUpdate()
//                    .set(CpAttachment::getPath,paAiTryon.getTargetUrl())
//                    .eq(CpAttachment::getId,paCreation.getPhotoId())
//                    .update();
//            paCreation.setState("t");
//            paCreationService.updateById(paCreation);
//        }catch (Exception e){
//            failureHandling(paCreation,e);
//        }
//    }

    @Override
    @Async
    @SysLog(action = "AiPicture:aiTryon", value = "阿里云：AI换衣")
    public void aiTryon(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //异步请求阿里云URL，获取任务id，tryon传参，上衣、下衣、人物
            String taskId=((Map)this.tryon(aiPictureVo,paCreation)).get("task_id").toString();
            paCreation.setTaskId(taskId);
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:portraitStyleRepainting", value = "阿里云：人像风格重绘")
    public void portraitStyleRepainting(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //异步请求阿里云URL，获取任务id
            String taskId=((Map)this.generation(aiPictureVo,paCreation)).get("task_id").toString();
            paCreation.setTaskId(taskId);
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:texture", value = "阿里云：艺术字")
    public void texture(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            //异步请求阿里云URL，获取任务id
            String taskId=((Map)this.textureImage(aiPictureVo,paCreation)).get("task_id").toString();
            paCreation.setTaskId(taskId);
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:HighAesSmartDrawing", value = "火山引擎：单张pv写真")
    public void HighAesSmartDrawing(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        IVisualService visualService = VisualServiceImpl.getInstance();
        visualService.setAccessKey(volcengineKEY);
        visualService.setSecretKey(volcengineSecret);
        //高美感通用v1.1-图生图
        JSONObject req=new JSONObject();
        req.put("req_key",aiPictureVo.getOption());
        req.put("prompt","");
        req.put("binary_data_base64",Arrays.asList(AliyunOssUtil.encodeToBase64(aiPictureVo.getImageUrl())));
        try {
            VisualHighAesSmartDrawingResponse response = visualService.visualHighAesSmartDrawing(req);
            if(response.getMessage().equals("Success")){
                //业务代码
                String base64=response.getData().getBinaryDataBase64().get(0);
                MultipartFile image=AliyunOssUtil.base64ToMultipartFile(base64,"temp");
                String imageUrl=AliyunOssUtil.uploadFile(image);
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(imageUrl);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("t");
                paCreationService.updateById(paCreation);
            }else{
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(fail);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("fail");
                paCreationService.updateById(paCreation);
            }
        } catch (Exception e) {
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:HumanSegmentDemo", value = "火山引擎：人像抠图")
    public void HumanSegmentDemo(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        IVisualService visualService = VisualServiceImpl.getInstance();

        visualService.setAccessKey(volcengineKEY);
        visualService.setSecretKey(volcengineSecret);

        VisualHumanSegmentRequest req = new VisualHumanSegmentRequest();
        req.setImageBase64(AliyunOssUtil.encodeToBase64(aiPictureVo.getImageUrl()));
        try {
            VisualHumanSegmentResponse response = visualService.humanSegment(req);
            if(response.getMessage().equals("Success")){
                //业务代码
                String base64=response.getData().getImage();
                MultipartFile image=AliyunOssUtil.base64ToMultipartFile(base64,"temp");
                String imageUrl=AliyunOssUtil.uploadFile(image);
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(imageUrl);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("t");
                paCreationService.updateById(paCreation);
            }else{
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(fail);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("fail");
                paCreationService.updateById(paCreation);
            }
        } catch (Exception e) {
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:ageChange", value = "火山引擎：年龄变换")
    public void ageChange(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            JSONObject req=new com.alibaba.fastjson.JSONObject();
            req.put("req_key", "all_age_generation");
            req.put("image_urls",Arrays.asList(aiPictureVo.getImageUrl()));
            req.put("target_age", Integer.valueOf(aiPictureVo.getOption()));
            req.put("do_risk", false);
            req.put("return_url", true);
            JSONObject response =VolcengineUtil.CVProcess(req);
            JSONObject data=response.getJSONObject("data");
            JSONArray imageUrls=data.getJSONArray("image_urls");
            String imageUrl=imageUrls.get(0).toString();
            //业务代码
            MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
            imageUrl=AliyunOssUtil.uploadFile(image);
            CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
            cpAttachment.setPath(imageUrl);
            cpAttachmentService.updateById(cpAttachment);
            paCreation.setState("t");
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }
    @Override
    @Async
    @SysLog(action = "AiPicture:jPCartoon", value = "火山引擎：人像漫画风")
    public void jPCartoon(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        IVisualService visualService = VisualServiceImpl.getInstance();
        visualService.setAccessKey(volcengineKEY);
        visualService.setSecretKey(volcengineSecret);

        VisualJPCartoonRequest req = new VisualJPCartoonRequest();
        req.setImageUrl(aiPictureVo.getImageUrl());
        req.setCartoonType(aiPictureVo.getOption());
        try {
            VisualJPCartoonResponse response = visualService.jpCartoon(req);
            if(response.getMessage().equals("Success")){
                //业务代码
                String base64=response.getData().getImage();
                MultipartFile image=AliyunOssUtil.base64ToMultipartFile(base64,"temp");
                String imageUrl=AliyunOssUtil.uploadFile(image);
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(imageUrl);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("t");
                paCreationService.updateById(paCreation);
            }else{
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(fail);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("fail");
                paCreationService.updateById(paCreation);
            }

        } catch (Exception e) {
            failureHandling(paCreation,e);
        }
    }
    @Override
    @Async
    @SysLog(action = "AiPicture:threeDGameCartoon", value = "火山引擎：3D游戏特效")
    public void threeDGameCartoon(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        IVisualService visualService = VisualServiceImpl.getInstance();
        visualService.setAccessKey(volcengineKEY);
        visualService.setSecretKey(volcengineSecret);

        VisualImg2Video3DRequest req = new VisualImg2Video3DRequest();
        req.setReqKey("img2video3d");
        ArrayList<String> binaryData = new ArrayList<>();
        binaryData.add(AliyunOssUtil.encodeToBase64(aiPictureVo.getImageUrl()));
        req.setBinaryDataBase64(binaryData);
        try {
            VisualImg2Video3DResponse response = visualService.img2Video3D(req);
            if(response.getMessage().equals("Success")){
                //业务代码
                String base64=response.getData().getBinaryDataBase64().get(0);
                MultipartFile image=AliyunOssUtil.base64ToMultipartFile(base64,"temp");
                String imageUrl=AliyunOssUtil.uploadFile(image);
                CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
                cpAttachment.setPath(imageUrl);
                cpAttachmentService.updateById(cpAttachment);
                paCreation.setState("t");
                paCreationService.updateById(paCreation);
            }
        } catch (Exception e) {
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:emotionPortrait", value = "火山引擎：表情编辑")
    public void emotionPortrait(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            JSONObject req=new JSONObject();
            req.put("req_key", "emotion_portrait");
            req.put("image_urls",Arrays.asList(aiPictureVo.getImageUrl()));
            req.put("target_emotion",aiPictureVo.getOption());
            req.put("return_url",true);
            JSONObject response =VolcengineUtil.CVProcess(req);
            JSONObject data=response.getJSONObject("data");
            JSONArray imageUrls=data.getJSONArray("image_urls");
            String imageUrl=imageUrls.get(0).toString();
            //业务代码
            MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
            imageUrl=AliyunOssUtil.uploadFile(image);
            CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
            cpAttachment.setPath(imageUrl);
            cpAttachmentService.updateById(cpAttachment);
            paCreation.setState("t");
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    @Override
    @Async
    @SysLog(action = "AiPicture:hairStyle", value = "火山引擎：发型编辑")
    public void hairStyle(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        try {
            JSONObject req=new JSONObject();
            req.put("req_key", "hair_style");
            req.put("image_urls",Arrays.asList(aiPictureVo.getImageUrl()));
            req.put("hair_type", Integer.valueOf(aiPictureVo.getOption()));
            req.put("return_url",true);
            JSONObject response =VolcengineUtil.CVProcess(req);
            JSONObject data=response.getJSONObject("data");
            JSONArray imageUrls=data.getJSONArray("image_urls");
            String imageUrl=imageUrls.get(0).toString();
            //业务代码
            MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
            imageUrl=AliyunOssUtil.uploadFile(image);
            CpAttachment cpAttachment=cpAttachmentService.getById(paCreation.getPhotoId());
            cpAttachment.setPath(imageUrl);
            cpAttachmentService.updateById(cpAttachment);
            paCreation.setState("t");
            paCreationService.updateById(paCreation);
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
    }

    private Map textureImage(AiPictureVo aiPictureVo,PaCreation paCreation){
        try {
            // 设置HTTP头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);//固定，返回类型，只有json
            headers.set("Authorization", API_KEY);//固定，api密钥
            headers.set("X-DashScope-Async", "enable");//只有默认，异步提交任务
            String url="https://dashscope.aliyuncs.com/api/v1/services/aigc/wordart/texture";//风格转换url
            Map<String, Object> text=new HashMap<>();/////////////////input中的text对象
            text.put("text_content",aiPictureVo.getText());//设置文字内容
//        text.put("font_name",aiPictureVo.getOption() );//设置文字样式
//        text.put("output_image_ratio",output_image_ratio);//设置图片中文字比例
            Map<String, Object> input = new HashMap<>();//放到request中
            input.put("text", text);//////////将text对象放进来
            input.put("prompt", aiPictureVo.getPromt());//////////提示词
            input.put("texture_style", aiPictureVo.getOption());//////////样式
            Map<String,Object> parameters=new HashMap<>();
            parameters.put("image_short_size",704);//默认尺寸
            parameters.put("n",1);//默认一张
            parameters.put("alpha_channel",false);//默认不带alpha通道
            Map<String, Object> request = new HashMap<>();//请求体
            request.put("model", "wordart-texture");/////////选择模型wordart-texture，一个apikey可以冲所有模型，详情看文档介绍
            request.put("input", input);/////////input放进来
            request.put("parameters", parameters);/////////parameters对象
            HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);//生成一个请求内容
            ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);//发送
            return (Map)response.getBody().get("output");
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
        return null;
    }
    public Map generation(AiPictureVo aiPictureVo, PaCreation paCreation) {
        try {
            // 设置HTTP头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);//固定，返回类型，只有json
            headers.set("Authorization", API_KEY);//固定，api密钥
            headers.set("X-DashScope-Async", "enable");//只有默认，异步提交任务
            String url="https://dashscope.aliyuncs.com/api/v1/services/aigc/image-generation/generation";//风格转换url
            Map<String, Object> input = new HashMap<>();//放到request中
            input.put("image_url", aiPictureVo.getImageUrl());//////////图片url
            // "https://c-ssl.duitang.com/uploads/item/201410/23/20141023222303_4UYNu.jpeg"测试用图
            input.put("style_index", Integer.valueOf(aiPictureVo.getOption()));//////////样式
            Map<String, Object> request = new HashMap<>();//请求体
            request.put("model", "wanx-style-repaint-v1");/////////选择模型wanx-style-repaint-v1，一个apikey可以冲所有模型，详情看文档介绍
            request.put("input", input);/////////文本描述和尺寸
            HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);//生成一个请求内容
            ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);//发送
            return (Map)response.getBody().get("output");
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
        return null;
    }

    public Map tryon(AiPictureVo aiPictureVo, PaCreation paCreation) {
        // 设置HTTP头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);
        headers.set("Authorization", "Bearer " + API_KEY); // 注意这里应该是Bearer token格式
        headers.set("X-DashScope-Async", "enable");

        // 构建请求体
        Map<String, Object> input = new HashMap<>();
        input.put("top_garment_url", aiPictureVo.getTopImageUrl());
        if (aiPictureVo.getBottomImageUrl()!=null)
            input.put("bottom_garment_url",aiPictureVo.getBottomImageUrl());
        input.put("person_image_url", aiPictureVo.getImageUrl());


        Map<String, Object> parameters = new HashMap<>();
        parameters.put("resolution", -1); // 确认这是API接受的有效值
        parameters.put("restore_face", true);

        Map<String, Object> request = new HashMap<>();
        request.put("model", "aitryon");
        request.put("input", input);
        request.put("parameters", parameters);
        String url = "https://dashscope.aliyuncs.com/api/v1/services/aigc/image2image/image-synthesis/";

        HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);

        try {
            ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);

            // 检查响应状态码是否为200 OK
            if (response.getStatusCode() == HttpStatus.OK) {
                return (Map)response.getBody().get("output");
            } else {
                throw new RuntimeException("无法获取返回: " + response.getStatusCode());
            }
        } catch (Exception e) {
            failureHandling(paCreation,e);
        }
        return null;
    }



    public Map text2Images(AiPictureVo aiPictureVo, PaCreation paCreation)  {
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);//固定，返回类型，只有json
            headers.set("Authorization", API_KEY);//固定，api密钥
            headers.set("X-DashScope-Async", "enable");//只有默认，异步提交任务
            String url="https://dashscope.aliyuncs.com/api/v1/services/aigc/text2image/image-synthesis";//文生图url
            Map<String, Object> input = new HashMap<>();//放到request中
            input.put("prompt", aiPictureVo.getText());//////////文本描述
            Map<String, Object> parameters = new HashMap<>();//放到request中
            parameters.put("style", aiPictureVo.getOption());//////////类型
            parameters.put("n", 1);//////////数量
            parameters.put("size", aiPictureVo.getSize());//////////尺寸
            Map<String, Object> request = new HashMap<>();//请求体
            request.put("model", "wanx-v1");/////////选择模型wanx-v1，一个apikey可以冲所有模型，详情看文档介绍
            request.put("input", input);/////////文本描述和尺寸
            request.put("parameters", parameters);//////////类型和数量
            HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);//生成一个请求内容
            ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);//发送
            return (Map)response.getBody().get("output");
        }catch (Exception e){
            failureHandling(paCreation,e);
        }
        return null;
    }

    public String getGeneration(String taskId) throws Exception {
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            ArrayList<Map<String,String>> list=(ArrayList<Map<String,String>>) ((Map) NewResponse.get("output")).get("results");
            url=list.get(0).get("url");
        }
        return url;
    }

    public String getTryonImageByTaskId(String taskId) throws Exception{
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            url=(String) ((Map) NewResponse.get("output")).get("image_url");//获取返回的阿里云云上的图片url
        }
        return url;
    }

    public String getText2ImageByTaskId(String taskId) throws Exception{
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            List<Map<String, String>> results=(List<Map<String, String>>) ((Map) NewResponse.get("output")).get("results");
            url=results.get(0).get("url");//获取返回的阿里云云上的图片url
        }
        return url;
    }



    public String basicCall(String prompt,String option) throws ApiException, NoApiKeyException {
        ImageSynthesisParam param =
                ImageSynthesisParam.builder()
                        .apiKey(API_KEY)
                        .model(ImageSynthesis.Models.WANX_V1)
                        .prompt(prompt)
                        .style(option)
                        .n(1)
                        .size("1024*1024")
                        .build();

        ImageSynthesis imageSynthesis = new ImageSynthesis();
        ImageSynthesisResult result = null;
        try {
            result = imageSynthesis.call(param);
        } catch (ApiException | NoApiKeyException e){
            throw new RuntimeException(e.getMessage());
        }
        return JsonUtils.toJson(result);
    }


    public DirVo getDir(Integer userId){
        // 生成新的文件名
        String timestamp = System.currentTimeMillis()+"";
        String fileNameSource=timestamp+"Source.png";
        String fileNameSource2=timestamp+"Source2.png";
        String fileNameSource3=timestamp+"Source3.png";
        String fileNameTarget=timestamp+"Target.png";

        //创建文件夹
        File fileDir =new File(externalResourceDir+"\\"+userId);
        if (!fileDir.exists() && !fileDir.isDirectory()) {
            fileDir.mkdirs();
        }

        //文件存储路径
        String pathSouce=externalResourceDir+"\\"+userId+"\\"+fileNameSource;
        String pathSouce2=externalResourceDir+"\\"+userId+"\\"+fileNameSource2;
        String pathSouce3=externalResourceDir+"\\"+userId+"\\"+fileNameSource3;
        String pathTarget=externalResourceDir+"\\"+userId+"\\"+fileNameTarget;
        DirVo dirVo=new DirVo(timestamp,fileNameSource,fileNameSource2,fileNameSource3,fileNameTarget,pathSouce,pathSouce2,pathSouce3,pathTarget);
        return dirVo;
    }

    public String getBaiduAiToken() throws IOException {
        OkHttpClient HTTP_CLIENT = new OkHttpClient().newBuilder().build();
        MediaType mediaType = MediaType.parse("application/json");
        RequestBody body = RequestBody.create(mediaType, "");
        Request request = new Request.Builder()
                .url("https://aip.baidubce.com/oauth/2.0/token?client_id=VLIGkTb90ycaA0RxgOwzAlFm&client_secret=zVbIP2i70weJ3vQ9NUqeindPMDW28KDl&grant_type=client_credentials")
                .method("POST", body)
                .addHeader("Content-Type", "application/json")
                .addHeader("Accept", "application/json")
                .build();
        Response response = HTTP_CLIENT.newCall(request).execute();
//        System.out.println(response.body().string());
        HashMap hashMap = JSON.parseObject(response.body().string(), HashMap.class);
        String token = hashMap.get("access_token").toString();
        return token;
    }



    @Override//图片转base64传给baiduapi解析后获取传回来的base64码,黑白图片上色
    public String ActivateMagic(String imgUrl,String url,String option) throws Exception {//转码传输到baiduapi
            String param;
            if (option!="")
                param =  "option=" + option +"&url=" + imgUrl;
            else
                param = "url=" + imgUrl;
            String accessToken =getBaiduAiToken();
            String result = HttpUtil.post(url, accessToken, param);

            // 解析JSON并提取image属性
            JSONObject jsonObject = JSONObject.parseObject(result);//测试
            String base64 = jsonObject.getString("image");
            return base64;
    }

    @Override//将转存好的base64码转为图片
     public String Base64ToImage(String outputImagePath,String base64String){
        // 输入 Base64 编码文件的路径
//        String outputImagePath =path+".png";
        try {
            byte[] imageBytes = Base64.getDecoder().decode(base64String);
            try (FileOutputStream fos = new FileOutputStream(outputImagePath)) {
                fos.write(imageBytes);
                System.out.println("图像已成功保存到 " + outputImagePath);
                return outputImagePath;
            }
        } catch (Exception e) {
            System.err.println("解码失败，可能是因为数据不完整或格式错误: " + e.getMessage());
        }
        return null;
    }


}
