package com.pixelai.api.picture.util;

import com.alibaba.fastjson.JSONObject;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.picture.service.AiPictureService;
import com.volcengine.service.visual.IVisualService;
import com.volcengine.service.visual.impl.VisualServiceImpl;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

@Data
@Component
public class  VolcengineUtil {
    private static String AccessKeyID;
    private static String SecretAccessKey;
    private static AiPictureService aiPictureService;

    @Value("${volcengine.AccessKeyID}")
    private  String AccessKeyID_temp;
    @Value("${volcengine.SecretAccessKey}")
    private  String SecretAccessKey_temp;
    @Autowired
    private AiPictureService aiPictureService_temp;


    //@Value无法注入static变量，使用临时变量赋值
    @PostConstruct
    public void init() {
        AccessKeyID=AccessKeyID_temp;
        SecretAccessKey=SecretAccessKey_temp;
        aiPictureService=aiPictureService_temp;
    }

    //同步
    public static JSONObject CVProcess(JSONObject req) {
        IVisualService visualService = VisualServiceImpl.getInstance();
        visualService.setAccessKey(AccessKeyID);
        visualService.setSecretKey(SecretAccessKey);

        try {
            Object response = visualService.cvProcess(req);
            return (JSONObject)response;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    //同步转异步
    public static JSONObject CVSubmitTask(PaCreation paCreation, JSONObject req) {
        IVisualService visualService = VisualServiceImpl.getInstance();
        visualService.setAccessKey(AccessKeyID);
        visualService.setSecretKey(SecretAccessKey);
        try {
            Object response = visualService.cvSync2AsyncSubmitTask(req);
            return (JSONObject)response;
        } catch (Exception e) {
            aiPictureService.failureHandling(paCreation,e);
        }
        return null;
    }

    //获取报错信息
    public static String getErrorMessage(Integer code) {
      String errorMessage = "获取原因失败";
      if (code == null) {
          return errorMessage;
      }
      switch(code) {
        case 50411:
            errorMessage="输入图片前审核未通过";
            break;
        case 50511:
            errorMessage="输出图片后审核未通过";
            break;
        case 50412:
              errorMessage="输入文本前审核未通过";
              break;
        case 50512:
              errorMessage="输出文本后审核未通过";
              break;
        case 50413:
              errorMessage="文本不符合安全规范，请调整文本内容";
              break;
        case 60101:
              errorMessage="图像解析错误";
              break;
        case 60102:
              errorMessage="图片中未找到人脸";
              break;
        case 60103:
              errorMessage="缺少输入图片";
              break;
        case 60106:
              errorMessage="输入图片损坏";
              break;
        case 60208:
              errorMessage="上传图片包含敏感信息";
              break;
        case 50205:
              errorMessage="图像尺寸超过限制";
              break;
        case 50429:
              errorMessage="当前使用人数过多，请稍后再试";
              break;
        case 50501:
              errorMessage="服务器内部RPC错误,请稍后再试";
              break;
        case 50400:
              errorMessage="权限校验失败，请联系管理员";
              break;
    }
        return errorMessage;
    }

}


