package com.pixelai.api.user.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.annotation.SysLog;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.user.entity.AcLoginlog;
import com.pixelai.api.user.service.AcLoginlogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author mzl
 * @since 2021-05-20
 */
@Api(tags = "登录日志")
@RestController
@RequestMapping("/acLoginlog")
public class AcLoginlogController extends BaseController {

    private static final Logger            LOG = LoggerFactory.getLogger(AcLoginlogController.class);
    @Autowired
    private              AcLoginlogService entityService;

    @SysLog(action = "findByPage", value = "分页查询登录日志")
    @ApiOperation(value = "分页查询登录日志", notes = "分页查询登录日志")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<AcLoginlog>> findByPage(final AcLoginlog example, final Page page) {
        IPage<AcLoginlog> records = entityService.page(example, page);
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除登录日志")
    @ApiOperation(value = "删除登录日志")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "登录日志ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        entityService.removeById(id);
        return new Result(true, "成功删除登录日志", null);
    }

    @SysLog(action = "one", value = "查询单个登录日志")
    @ApiOperation(value = "查询单个登录日志")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "登录日志ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        AcLoginlog entity = entityService.getById(id);
        return new Result(true, entity);
    }

    @SysLog(action = "add", value = "添加登录日志")
    @ApiOperation(value = "添加登录日志")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final AcLoginlog entity) {
        entityService.save(entity);
        return new Result(true, "成功保存登录日志", null);
    }

    @SysLog(action = "update", value = "修改登录日志")
    @ApiOperation(value = "修改登录日志")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final AcLoginlog entity) {
        if (null == entity.getId()) {
            throw new RuntimeException("id不能为空");
        }
        entityService.updateById(entity);
        return new Result(true, "成功修改登录日志", null);
    }

    @ApiOperation(value = "全部登录日志")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<AcLoginlog>> all(AcLoginlog example) {
        List<AcLoginlog> entitys = entityService.list(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}

