package com.pixelai.api.user.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.annotation.SysLog;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.user.entity.AcUsertype;
import com.pixelai.api.user.service.AcUsertypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author mzl
 * @since 2020-07-09
 */
@Api(tags = "用户类型")
@RestController
@RequestMapping("/acUsertype")
public class AcUsertypeController extends BaseController {

    private static final Logger            LOG = LoggerFactory.getLogger(AcUsertypeController.class);
    @Autowired
    private              AcUsertypeService entityService;

    @SysLog(action = "acUsertype:findByPage", value = "用户管理：用户类型列表")
    @ApiOperation(value = "分页查询用户类型", notes = "分页查询用户类型")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<AcUsertype>> findByPage(final AcUsertype example, final Page page) {
        IPage<AcUsertype> records = entityService.page(example, page);
        return new Result(true, records);
    }

    @SysLog(action = "acUsertype:delete", value = "用户管理：删除用户类型")
    @ApiOperation(value = "删除用户类型")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "用户类型ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        entityService.removeById(id);
        return new Result(true, "成功删除用户类型", null);
    }

    @SysLog(action = "acUsertype:one", value = "用户管理：查询用户类型详细")
    @ApiOperation(value = "查询单个用户类型")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "用户类型ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        AcUsertype entity = entityService.getById(id);
        return new Result(true, entity);
    }

    @SysLog(action = "acUsertype:add", value = "用户管理：添加用户类型")
    @ApiOperation(value = "添加用户类型")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final AcUsertype entity) {
        entityService.save(entity);
        return new Result(true, "成功保存用户类型", null);
    }

    @SysLog(action = "acUsertype:update", value = "用户管理：修改用户类型")
    @ApiOperation(value = "修改用户类型")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final AcUsertype entity) {
        if (null == entity.getId()) {
            throw new RuntimeException("id不能为空");
        }
        entityService.updateById(entity);
        return new Result(true, "成功修改用户类型", null);
    }

    @SysLog(action = "acUsertype:all", value = "用户管理：全部用户类型")
    @ApiOperation(value = "全部用户类型")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<AcUsertype>> all(AcUsertype example) {
        List<AcUsertype> entitys = entityService.list(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }



}
